const int MAXNUM = 1000;
struct SegTreeNode
{
  int val;
  int addMark;// label retardé
} segTree[MAXNUM];

/*
arr: valeur des feuilles
istart: indice début
iend: indice end
*/
void build(int root, int arr[], int istart, int iend)
{
  segTree[root].addMark = 0;
  if(istart == iend) // attend un feuille
    segTree[root].val = arr[istart];
  else
  {
    int mid = (istart + iend) / 2;
    build(root*2+1, arr, istart, mid); // créé l'arbre gauche
    build(root*2+2, arr, mid+1, iend); // créé l'arbre droit
    // calcul le min des deux
    segTree[root].val = min(segTree[root*2+1].val, segTree[root*2+2].val);
  }
}
/*
pushDown le label retardé
*/
void pushDown(int root)
{
  if(segTree[root].addMark != 0)
  {
    segTree[root*2+1].addMark += segTree[root].addMark;
    segTree[root*2+2].addMark += segTree[root].addMark;
    segTree[root*2+1].val += segTree[root].addMark;
    segTree[root*2+2].val += segTree[root].addMark;
    segTree[root].addMark = 0;
  }
}
/*
[nstart, nend]: intervale associé à root
[qstart, qend]: intervale de la question
*/
int query(int root, int nstart, int nend, int qstart, int qend)
{
  // pas d'intersection
  if(qstart > nend || qend < nstart)
    return INFINITE;
  // [nstart, nend] est inclu dans [qstart, qend]
  if(qstart <= nstart && qend >= nend)
    return segTree[root].val;
  pushDown(root);
  // min de la question sur les deux fils
  int mid = (nstart + nend) / 2;
  return min(query(root*2+1, nstart, mid, qstart, qend),
           query(root*2+2, mid + 1, nend, qstart, qend));

}

/*
change la valeur d'une feuille
[nstart, nend]
index: indice de la feuille dans arr
addVal: différence à ajouter
*/
void updateOne(int root, int nstart, int nend, int index, int addVal)
{
  if(nstart == nend)
  {
      if(index == nstart) // feuille cherchée
          segTree[root].val += addVal;
      return;
  }
  int mid = (nstart + nend) / 2;
  if(index <= mid) // à gauche
      updateOne(root*2+1, nstart, mid, index, addVal);
  else updateOne(root*2+2, mid+1, nend, index, addVal); // à droite
  // prend le minimum
  segTree[root].val = min(segTree[root*2+1].val, segTree[root*2+2].val);
}

/*
Met à jour un intervale
[nstart, nend]: intervale du noeud
[ustart, uend]: intervale à mettre à jour
addVal: différence à ajouter
*/
void update(int root, int nstart, int nend, int ustart, int uend, int addVal)
{
  // pas d'intersection
  if(ustart > nend || uend < nstart)
    return;
  // inclu
  if(ustart <= nstart && uend >= nend)
  {
    segTree[root].addMark += addVal;
    segTree[root].val += addVal;
    return;
  }
  pushDown(root);
  int mid = (nstart + nend) / 2;
  update(root*2+1, nstart, mid, ustart, uend, addVal);
  update(root*2+2, mid+1, nend, ustart, uend, addVal);
  segTree[root].val = min(segTree[root*2+1].val, segTree[root*2+2].val);
}
